package org.japura.task;

import java.util.List;

/**
 * <P>
 * Copyright (C) 2012 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public class TaskSequence{

  private Task<?> firstTask;
  private Task<?> lastTask;
  private String message;

  public TaskSequence() {}

  public TaskSequence(List<Task<?>> tasks) {
	for (Task<?> task : tasks) {
	  add(task);
	}
  }

  public TaskSequence(Task<?> tasks[]) {
	for (Task<?> task : tasks) {
	  add(task);
	}
  }

  public String getMessage() {
	return message;
  }

  public void setMessage(String message) {
	this.message = message;
	updateFirstTaskMessage();
  }

  private void updateFirstTaskMessage() {
	if (getFirstTask() != null && getFirstTask().getMessage() == null) {
	  getFirstTask().setMessage(getMessage());
	}
  }

  public final void add(TaskSequence sequence) {
	if (firstTask == null) {
	  firstTask = sequence.getFirstTask();
	  updateFirstTaskMessage();
	} else {
	  lastTask.addNestedTask(sequence.getFirstTask());
	}
	lastTask = sequence.getLastTask();
  }

  public final void add(Task<?> task) {
	if (firstTask == null) {
	  firstTask = task;
	  updateFirstTaskMessage();
	} else {
	  lastTask.addNestedTask(task);
	}
	lastTask = task;
  }

  Task<?> getLastTask() {
	return lastTask;
  }

  public final Task<?> getFirstTask() {
	return firstTask;
  }

  public final boolean hasTask() {
	if (getFirstTask() != null) {
	  return true;
	}
	return false;
  }

}
